# coding: utf-8
# Названия методов

## Окончания: ?, !, =
## ? - метод, который возвращает логическое значение. Желательно использовать в коде.
def good_morning?
  false
end
## ! - изменяет состояние объекта
sample = 'TeSTIng the method'
p sample.capitalize
p sample
p sample.capitalize!
p sample
## Либо объект изменяемый, либо нет. Логично использовать для библиотек.

## = - возможность использования метода для присваивания значения. Для создания виртуальных атрибутов.
class Element
  def color=(color)
    p color
  end
end
element = Element.new
element.color = 'testing'

# Аргументы методов

## Нет аргументов
def method_without_arguments
  # body
end

## Список обязательных аргументов. Для вызова метода НАДО передать ровно 3 аргумента.
def method_with_some_arguments(arg1, arg2, arg3); end

## Аргументы могут иметь значение по умолчанию
def cool_dude(arg1 = 'Petya', arg2 = 'Vasya', arg3 = 'Kolya')
  puts "#{arg1}, #{arg2}, #{arg3}"
end
cool_dude
cool_dude('Andrey', 'Dasha')
cool_dude('Abc', nil, 'Doll')
## Никогда так не делайте!
def default_first(arg1 = 'Fox', arg2)
  puts "#{arg1}, #{arg2}"
end
default_first("test")
## Вы можете использовать значение раннего аргумента как значение позднего аргумента
def surrond(word, pad_width = word.length / 2)
  '[' * pad_width + word + ']' * pad_width
end
puts surrond('elephant')
puts surrond('fox')
puts surrond('dog', 5)

## Методы с произвольным количеством аргументов
def varargs(arg1, *rest)
  puts "arg1=#{arg1}, rest=#{rest.inspect}"
end
varargs('one')
varargs('one', 'two')
varargs('one', 'two', 'three')
## Для передачи произвольного количества аргументов используйте *
varargs(*['One', 'Two'], 'Three', *(44..55))

## Магия с вызвом super
class Parent
  def method(one, two)
    puts "#{one}, #{two}"
  end
end
class Child < Parent
  def method(*)
    super
    puts 'Going well'
  end
end
child = Child.new
child.method('one', 'two')
## Можно ли собрать произовльное количество аргументов "в середине"
def splat_apart(first, *splat, last)
  puts "First: #{first}, splat: #{splat.inspect}, last: #{last}"
end
splat_apart('one', 'two')
splat_apart('one', 'two', 'three')
## Можно ли указать несколько переменных для сбора произвольного количества аргументов?
# Работать не будет!
# def test_method(*first, *second)
# end
## Строго именованные аргументы
### История )
def seach_song(title, options = {})
  # Используем options...
end
### Бывает, что вы хотите удостовериться, что вам передали ТОЛЬКО поддерживаемые параметры
### а также нужны значения по умолчанию
def search(field, options = {})
  options = { duration: 120 }.merge(options)
  if options.has_key?(:duration)
    duration = options[:duration]
  end
  options.delete(:duration)
  if options.has_key?(:genre)
    genre = options[:genre]
  end
  options.delete(:genre)
  fail "Invalid options passed: #{options.keys.join(', ')}" unless options.empty?
end
### Стоит отметить, что ключами хеша являются символы.
### В Ruby 2.0 ввели специальную поддержку для такого рода аргументов
def search_new(field, genre: nil, duration: 120)
  p [field, genre, duration]
end
search_new(:title)
# Ошибка, т.к. опции gegre не существует
# search_new(:title, gegre: 'rock', duration: 500)
search_new(:title, duration: 600)
search_new(:title, genre: 'abc')
search_new(:title, genre: 'rock', duration: 500)
search_new(:title, { genre: 'soft' })
# Ошибка, т.к. ключевого слова rock не существует
# search_new(:title, { genre: 'soft', rock: true })
# Ошибка, т.к. передаётся 3 обязательных агрумента вместо 1
# search_new(:description, 'rock', 500)

## Произвольное количество аргументов в хеше
def mega_search(field, genre: nil, duration: 120, **rest)
  p [field, genre, duration, rest]
end
mega_search(:title, gegre: 'rock', duration: 500)
